package com.starbuds.server.service.net.utils;

import com.starbuds.server.common.pojo.net.NetResponse;
import kotlin.Pair;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @author aoe
 * @date 2020/6/10
 */
@Slf4j
public class HttpUtil {

    /**
     * 跟踪Id
     */
    private final static String TRACE_ID = "xyTraceID";

    /**
     * 是否同步请求
     */
    private final static String TRACE_IS_SYNC = "xyTraceIsSync";

    public static NetResponse getNetResponse(Response res) {
        if (res == null) {
            return null;
        }

        int code = res.code();
        NetResponse netRes = new NetResponse();
        netRes.setStatus(code);
        netRes.setSuccess(code == 200);

        if (res.headers() != null && res.headers().size() > 0) {
            Map<String, String> resHeaders = new HashMap<>(res.headers().size());
            for (Pair<? extends String, ? extends String> pair : res.headers()) {
                resHeaders.put(pair.getFirst(), pair.getSecond());
            }
            netRes.setHeaders(resHeaders);
        }

        ResponseBody body = res.body();
        if (body != null) {
            try {
                netRes.setBody(body.string());
            } catch (IOException e) {
                log.error("ResponseBody 转 String 异常", e);
            }
        }

        return netRes;
    }

    /**
     * 设置跟踪信息
     *
     * @param headers
     * @param isSync
     * @return
     */
    public static Map<String, String> setTraceInfo(Map<String, String> headers, boolean isSync) {
        if (headers == null) {
            headers = new HashMap<>(1);
        }
        headers.put(TRACE_ID, UUID.randomUUID().toString());
        headers.put(TRACE_IS_SYNC, "" + isSync);
        return headers;
    }

    /**
     * 获得跟踪信息Id
     *
     * @param headers
     * @return
     */
    public static String getTraceId(Map<String, String> headers) {
        if (headers == null) {
            return null;
        }
        return headers.get(TRACE_ID);
    }

    /**
     * 获得跟踪信息Id
     *
     * @param request
     * @return
     */
    public static String getTraceId(Request request) {
        if (request == null) {
            return null;
        }
        return request.header(TRACE_ID);
    }

    /**
     * 获得是否同步请求
     *
     * @param request
     * @return
     */
    public static boolean getTraceIsSync(Request request) {
        if (request == null) {
            log.warn("request 为 null, 是否同步请求返回 false ");
            return false;
        }
        return "true".equals(request.header(TRACE_IS_SYNC));
    }

    /**
     * 清空请求跟踪信息
     *
     * @param request
     * @return
     */
    public static Request cleanTraceInfo(Request request) {
        if (request.headers() == null || request.headers().size() < 1) {
            return request;
        }

        Request.Builder builder = request.newBuilder();
        builder.removeHeader(TRACE_ID);
        builder.removeHeader(TRACE_IS_SYNC);
        return builder.build();
    }

}
