package com.starbuds.server.service.net.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.fasterxml.jackson.dataformat.xml.annotation.JacksonXmlProperty;
import com.fasterxml.jackson.dataformat.xml.annotation.JacksonXmlRootElement;
import com.starbuds.BaseTest;
import com.starbuds.server.common.pojo.net.NetResponse;
import com.starbuds.server.common.service.net.RequestService;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
public class RequestServiceImplTest extends BaseTest {

    private String url = "http://192.168.0.117:8080/";

    @Reference(version = "1.0.0")
    private RequestService requestService;

    @Test
    public void syncGetTest() {
        NetResponse res = requestService.syncGet(url + "getTest?msg=rrrrr");
        checkSuccess(res);
    }

    @Test
    public void asyncGetTest() {
        boolean res = requestService.asyncGet(url + "getTest?msg=rrrrr");
        checkSuccess(res);
    }

    @Test
    public void syncGetHeadersTest() {
        NetResponse res = requestService.syncGet(url + "getTest?msg=rrrrr", getHeaders());
        checkSuccess(res);
    }

    @Test
    public void asyncGetHeadersTest() {
        boolean res = requestService.asyncGet(url + "getTest?msg=rrrrr", getHeaders());
        checkSuccess(res);
    }

    @Test
    public void syncGetHeadersTimoutTest() {
        NetResponse res = requestService.syncGet(url + "getTest?msg=rrrrr", getTimeoutHeaders(), 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    @Test
    public void asyncGetHeadersTimoutTest() {
        boolean res = requestService.asyncGet(url + "getTest?msg=rrrrr", getTimeoutHeaders(), 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    @Test
    public void syncPostTest() {
        Map<String, String> params = new HashMap<>();
        params.put("name", "名称");
        params.put("userId", "2101910");
        NetResponse res = requestService.syncPost(url + "postTest", params);
        checkSuccess(res);
    }

    @Test
    public void asyncPostTest() {
        Map<String, String> params = new HashMap<>();
        params.put("name", "名称");
        params.put("userId", "2101910");
        boolean res = requestService.asyncPost(url + "postTest", params);
        checkSuccess(res);
    }

    @Test
    public void syncPostHeaderTest() {
        Map<String, String> params = new HashMap<>();
        params.put("name", "名称");
        params.put("userId", "2101910");
        NetResponse res = requestService.syncPost(url + "postTest", getHeaders(), params);
        checkSuccess(res);
    }

    @Test
    public void asyncPostHeaderTest() {
        Map<String, String> params = new HashMap<>();
        params.put("name", "名称");
        params.put("userId", "2101910");
        boolean res = requestService.asyncPost(url + "postTest", getHeaders(), params);
        checkSuccess(res);
    }

    @Test
    public void syncPostHeaderTimoutTest() {
        Map<String, String> params = new HashMap<>();
        params.put("name", "名称");
        params.put("userId", "2101910");
        NetResponse res = requestService.syncPost(url + "postTest", getTimeoutHeaders(), params, 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    @Test
    public void asyncPostHeaderTimoutTest() {
        Map<String, String> params = new HashMap<>();
        params.put("name", "名称");
        params.put("userId", "2101910");
        boolean res = requestService.asyncPost(url + "postTest", getTimeoutHeaders(), params, 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    @Test
    public void syncPostJsonTest() {
        JSONObject json = new JSONObject();
        json.put("name", "名称");
        json.put("userId", "2101910");
        NetResponse res = requestService.syncPost(url + "postJsonTest", getTimeoutHeaders(), json.toJSONString(), 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    @Test
    public void asyncPostJsonTest() {
        JSONObject json = new JSONObject();
        json.put("name", "名称");
        json.put("userId", "2101910");
        boolean res = requestService.asyncPost(url + "postJsonTest", getTimeoutHeaders(), json.toJSONString(), 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    @Test
    public void syncPostXmlTest() throws JsonProcessingException {
        XmlRequest xml = new XmlRequest();
        xml.setId(39000);
        xml.setName("xml 请求");
        xml.setShortName("段明珠");

        ObjectMapper xmlMapper = new XmlMapper();
        String xmlStr = xmlMapper.writeValueAsString(xml);

        NetResponse res = requestService.syncPost(url + "postXmlTest", getTimeoutHeaders(), xmlStr, 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    @Test
    public void syncPostFileTest() {
        JSONObject json = new JSONObject();
        json.put("name", "名称");
        json.put("userId", "2101910");
        NetResponse res = requestService.syncPost(url + "postJsonTest", getTimeoutHeaders(), json.toJSONString(), 5000L, TimeUnit.SECONDS);
        checkSuccess(res);
    }

    private void checkSuccess(NetResponse res) {
        log.info("res\n{}", res);
        Assert.assertEquals(res.getStatus(), 200);
        Assert.assertEquals(res.isSuccess(), true);
    }

    private void checkSuccess(boolean res) {
        log.info("res\n{}", res);
        Assert.assertEquals(res, true);
    }

    private Map<String, String> getHeaders() {
        Map<String, String> headers = new HashMap<>();
        headers.put("local", "Shanghai");
        headers.put("fruit", "Mango");
        return headers;
    }

    private Map<String, String> getTimeoutHeaders() {
        Map<String, String> headers = getHeaders();
        headers.put("timout", "5000");
        return headers;
    }

    @JacksonXmlRootElement(localName = "school")
    @Data
    private class XmlRequest {
        @JacksonXmlProperty(localName = "id")
        private Integer id;
        @JacksonXmlProperty(localName = "name")
        private String name;
        @JacksonXmlProperty(localName = "shortname")
        private String shortName;
    }
}